// Trampa de velocidad con Arduino
// (Medidor de velocidad para ferrocarriles a escala)
// Dario Calvo Sanchez, 2017-2020
// Trampa de velocidad con Arduino by Dario Calvo Sanchez is licensed under CC BY-NC-SA 4.0. To view a copy of this license, visit https://creativecommons.org/licenses/by-nc-sa/4.0
// Uses library "SevenSegmentTM1637.h" by Bram Harmsen (https://github.com/bremme/arduino-tm1637)

// Version para distancia entre reeds de 25 cm y escala H0 por defecto

// Libreria para display LED TM1637 de cuadro digitos y siete segmentos
// Pin CLK: 2
// Pin I/O: 3
#include <SevenSegmentTM1637.h>
SevenSegmentTM1637 display(2, 3); // Definicion de la clase del display LED

// Definicion de variables:
unsigned long tinicial; // Entero largo para los milisegundos del sistema
unsigned long tfinal; // Entero largo para los milisegundos del sistema
float transcurrido;
float segundos;
float  distancia_cm = 25.00; // Distancia entre reeds (valor fijo)
float velocidad_real;
int factor_escala = 87; // Valor de escala inicial ajustado a H0 (1:87)
int velocidad_escala;
int contador1 = 0; // Valor inicial del contador de control de flujo para el reed 1: 0
int contador2 = 0; // Valor inicial del contador de control de flujo para el reed 1: 0
int contadorDisp = 0; // Valor inicial del contador de control de flujo para el display: 0
int displayoffset = 0; // Valor inicial de la posicion del cursor del display, contando desde el extremo izquierdo

// Definicion de pines:
// Pin del primer reed: 5
// Pin del segundo reed: 6
// Pin del selector de escala: 8
int reed1 = 5;
int reed2 = 6;
int selector_escala = 8;

// Configuracion del dispositivo:
void setup(){
 Serial.begin(9600); // Inicializa consola en el puerto serie, 9600 baudios
 display.begin(); // Inicializa display LED
 display.setBacklight(100); // Ajusta brillo del display LED al 100%
 pinMode(reed1, INPUT_PULLUP);
 digitalWrite(reed1, HIGH); // Configuracion inicial del boton del segundo reed en "HIGH", al estar en configuracion "pull-up"
 pinMode(reed2, INPUT_PULLUP);
 digitalWrite(reed2, HIGH); // Configuracion inicial del boton del segundo reed en "HIGH", al estar en configuracion "pull-up"
 pinMode(selector_escala, INPUT_PULLUP);
 digitalWrite(selector_escala, HIGH); // Configuracion inicial del boton de reinicio en "HIGH", al estar en configuracion "pull-up"
 
 Serial.println(contador1);
 Serial.println(contador2);
 Serial.println("OK");
 display.print("0000"); // Muestra "0000" en el display para indicar el arranque...
 display.blink(); // ...lo hace parpadear...
 delay(500); // ...espera 500 ms...
 display.clear(); // ...y lo apaga
 delay(500); // Espera 500 ms
 display.print("E"); // Escribe una "E" de "escala"
 display.setCursor(1, 1); // Avanza el cursor una posicion para no "pisar" la "E"
 display.print(factor_escala); // Muestra en el display el factor de escala por defecto...
 delay(1000); // ...espera 1000 ms...
 display.home(); // ...devuelve el cursor a su sitio...
 display.clear(); // ...y lo limpia
}

// Bucle principal:
void loop(){
 // Seleccion de escala:
 if((contador1 == 0) && (contador2 == 0)) { // Solo permite hacerlo con el sistema en reposo
     if((digitalRead(selector_escala) == LOW) && (factor_escala == 160)){ // Si se ha pulsado el boton y el factor esta en escala N (1:160)...
      delay(500); // ...espera 500 ms para no leer varias veces la señal...
      factor_escala = 87; // ...y cambia a escala H0 (1:87)
      display.print("E"); // Escribe una "E" de "escala"
      display.setCursor(1, 2); // Avanza el cursor dos posiciones para no "pisar" la "E"
      display.print(factor_escala); // Muestra en el display el factor de escala por defecto...
      delay(1000); // ...espera 1000 ms...
      display.home(); // ...devuelve el cursor a su sitio...
      display.clear(); // ...y lo limpia
     }
     if((digitalRead(selector_escala) == LOW) && (factor_escala == 87)){ // Si se ha pulsado el boton y el factor esta en escala H0 (1:87)...
      delay(500); // ...espera 500 ms para no leer varias veces la señal...
      factor_escala = 160; // ...y cambia a escala N (1:160)
      display.print("E"); // Escribe una "E" de "escala"
      display.setCursor(1, 1); // Avanza el cursor una posicion para no "pisar" la "E"
      display.print(factor_escala); // Muestra en el display el factor de escala por defecto...
      delay(1000); // ...espera 1000 ms...
      display.home(); // ...devuelve el cursor a su sitio...
      display.clear(); // ...y lo limpia
     }
   }
 // Comprobacion de contactos: si toca en reed 2...
 if(digitalRead(reed2) == LOW){
     if((contador1 == 1) && (contador2 == 0)){ // O sea, si ya ha pasado por el otro contacto reed pero no por este...
      tfinal = millis(); // Utiliza el reloj del sistema como base: toma como tiempo final el valor de los milisegundos de funcionamiento del sistema cuando se activa el segundo reed
      transcurrido = float(tfinal - tinicial); // Tiempo transcurrido desde que se activo el primer reed hasta que lo ha hecho el segundo. Ya lo almacena como punto flotante con la funcion float()
      segundos = transcurrido / 1000.; // No olvidar el punto despues del "1000" para asegurarnos que toma la constante como "float"
      contador2 = 1; // Valor del contador a "1" para indicar el final de la toma de tiempos
      Serial.println(contador2);
     }
     if((contador1 == 0) && (contador2 == 0)){ // No habia contacto previo: es el primer contacto, inicia el cronometro y pone el contador en "1" para que al pasar sobre el siguiente reed lo "detenga" y calcule
      tinicial = millis(); // Utiliza el reloj del sistema como base: toma como origen de tiempos el valor de los milisegundos de funcionamiento del sistema cuando se activa el primer reed
      contador2 = 1; // Valor del contador a 1 para indicar el primer paso por el reed
      Serial.println(contador2);
     }
   }
 // ...y si toca en el reed 1
 if(digitalRead(reed1) == LOW){
     if((contador2 == 1) && (contador1 == 0)){ // O sea, si ya ha pasado por el otro contacto reed pero no por este...
      tfinal = millis(); // Utiliza el reloj del sistema como base: toma como tiempo final el valor de los milisegundos de funcionamiento del sistema cuando se activa el segundo reed
      transcurrido = float(tfinal - tinicial); // Tiempo transcurrido desde que se activo el primer reed hasta que lo ha hecho el segundo. Ya lo almacena como punto flotante con la funcion float()
      segundos = transcurrido / 1000.; // No olvidar el punto despues del "1000" para asegurarnos que toma la constante como "float"
      contador1 = 1; // Valor del contador a "1" para indicar el final de la toma de tiempos
      Serial.println(contador1);
     }
     if((contador2 == 0) && (contador1 == 0)){ // No habia contacto previo: es el primer contacto, inicia el cronometro y pone el contador en "1" para que al pasar sobre el siguiente reed lo "detenga" y calcule
      tinicial = millis(); // Utiliza el reloj del sistema como base: toma como origen de tiempos el valor de los milisegundos de funcionamiento del sistema cuando se activa el primer reed
      contador1 = 1; // Valor del contador a 1 para indicar el primer paso por el reed
      Serial.println(contador1);
     }
   }
 // Calculo de la velocidad:
 if((contador1 == 1) && (contador2 == 1)){
  velocidad_real = distancia_cm / segundos; // En cm/s, en numero real (punto flotante)
  velocidad_escala = int(factor_escala * 0.036 * velocidad_real); // En km/h (gracias al factor de 0,036) y a escala (gracias al factor de escala). Convertido a int con la funcion int()
    if(velocidad_escala > 9999 || velocidad_escala < 0){ // Por si sale del limite del display o si por alguna razon sale negativo
      Serial.println("ERR");
      display.clear(); // Limpia el display
      display.print("ERR");
    }
    else{
    // Salida a la consola:
    Serial.print(transcurrido);
    Serial.println(" ms (redondeado a dos decimales por la funcion Serial.print)");
    Serial.print(segundos);
    Serial.println(" s (redondeado a dos decimales por la funcion Serial.print)");
    Serial.print(distancia_cm);
    Serial.println(" cm (redondeado a dos decimales por la funcion Serial.print)");
    Serial.print(velocidad_real);
    Serial.println(" cm/s (redondeado a dos decimales por la funcion Serial.print)");
    Serial.print("v = ");
    Serial.print(velocidad_escala);
    Serial.print(" km/h a escala 1:");
    Serial.println(factor_escala);
    
    // Salida al display LED:
    if((velocidad_escala >= 100) && (velocidad_escala < 1000)){ // Calcula la posicion del cursor para que todo quede alineado a la derecha
      displayoffset = 1;
    }
    if((velocidad_escala >= 10) && (velocidad_escala < 100)){
      displayoffset = 2;
    }
    if((velocidad_escala >= 0) && (velocidad_escala < 10)){
      displayoffset = 3;
    }
    display.clear(); // Limpia el display
    display.setCursor(1, displayoffset); // Avanza el cursor las posiciones necesarias para alinear el valor a la derecha...
    display.print(velocidad_escala); // ...muestra la velocidad en el display LED...
    display.home(); // ...y devuelve el cursor a su sitio
    }
  delay(2000); // Espera 2000 ms como medida de precaucion para evitar falsos contactos de otras zonas del tren y...
  displayoffset = 0;
  contador1 = 0;
  contador2 = 0; // ...deja listo el reloj para la siguiente medicion
  display.clear(); // Limpia el display
 }
}
